#include "no_rtti_shapes.h"

// Straightforward Shape-derivative that draws itself, can be serialized, is selectable, movable, and deletable
class Triangle :
	public Shape
{
public:
	Triangle() :
		m_X(0),
		m_Y(0),
		m_Selected(false)
	{
	}
	
	~Triangle()
	{
		std::cout << "Triangle::~Triangle()" << std::endl;
	}
	
	bool IsDrawable()
	{
		return true;
	}
	
	void Draw()
	{
		// Triangle-specific drawing code here
		std::cout << "Triangle::Draw()" << std::endl;
	}
	
	bool IsSerializable()
	{
		return true;
	}
	
	void Save(std::ostream& Stream)
	{
		// Triangle-specific serialization code here
		std::cout << "Triangle::Save()" << std::endl;
	}
	
	bool IsSelectable()
	{
		return true;
	}
	
	void Select()
	{
		m_Selected = true;
	}
	
	void Deselect()
	{
		m_Selected = false;
	}

	bool IsSelected()
	{
		return m_Selected;
	}
	
	bool IsMovable()
	{
		return true;
	}
	
	void Move(const double X, const double Y)
	{
		m_X = X;
		m_Y = Y;
	}
	 
	bool IsDeletable()
	{
		return true;
	}

private:
	double m_X;
	double m_Y;
	bool m_Selected;
};

// Specialized Shape-derivative that draws itself, can be serialized, is selectable and deletable, but can't be moved
class PageBorder :
	public Shape
{
public:
	PageBorder() :
		m_Selected(false)
	{
	}
	
	~PageBorder()
	{
		std::cout << "PageBorder::~PageBorder()" << std::endl;
	}
	
	bool IsDrawable()
	{
		return true;
	}
	
	void Draw()
	{
		// Border-specific drawing code here
		std::cout << "PageBorder::Draw()" << std::endl;
	}
	
	bool IsSerializable()
	{
		return true;
	}
	
	void Save(std::ostream& Stream)
	{
		// Border-specific serialization code here
		std::cout << "PageBorder::Save()" << std::endl;
	}
	
	bool IsSelectable()
	{
		return true;
	}
	
	void Select()
	{
		m_Selected = true;
	}
	
	void Deselect()
	{
		m_Selected = false;
	}

	bool IsSelected()
	{
		return m_Selected;
	}
	
	bool IsMovable()
	{
		return false;
	}
	
	void Move(const double X, const double Y)
	{
		// Should never be called
	}
	 
	bool IsDeletable()
	{
		return true;
	}

private:
	bool m_Selected;
};

// Special application-managed shape that is drawable only
class Ruler :
	public Shape
{
public:
	Ruler()
	{
	}
	
	~Ruler()
	{
		std::cout << "Ruler::~Ruler()" << std::endl;
	}
	
	bool IsDrawable()
	{
		return true;
	}
	
	void Draw()
	{
		// Ruler-specific drawing code here
		std::cout << "Ruler::Draw()" << std::endl;
	}
	
	bool IsSerializable()
	{
		return false;
	}
	
	void Save(std::ostream& Stream)
	{
		// Should never be called
		std::cout << "Ruler::Save()" << std::endl;
	}
	
	bool IsSelectable()
	{
		return false;
	}
	
	void Select()
	{
		// Should never be called
	}
	
	void Deselect()
	{
		// Should never be called
	}

	bool IsSelected()
	{
		// This should never be called, but we have to return something;
		// this is one of the serious flaws with this design
		return false;
	}
	
	bool IsMovable()
	{
		return false;
	}
	
	void Move(const double X, const double Y)
	{
		// Should never be called
	}
	 
	bool IsDeletable()
	{
		return false;
	}
};

int main(int argc, char* argv[])
{
	// Application-managed objects that aren't user-deletable
	Ruler* const vertical_ruler = new Ruler();
	Ruler* const horizontal_ruler = new Ruler();

	// Created when the user starts a new drawing
	Canvas* const canvas = new Canvas();
	
	// Default shapes that apply to every canvas
	canvas->AddShape(vertical_ruler);
	canvas->AddShape(horizontal_ruler);

	// Simulate adding  several shapes to the canvas by the user
	Shape* const user1 = new Triangle();
	canvas->AddShape(user1);
	Shape* const user2 = new Triangle();
	canvas->AddShape(user2);
	Shape* const user3 = new PageBorder();
	canvas->AddShape(user3);

	// Uncomment this to see compile-time protection against assignment of objects
	// *user1 = *user3;
	
	// Uncomment this to test compile-time protection against destruction of objects through interfaces
	// delete user2;
	
	// Simulate a normal screen-refresh event	
	canvas->Draw();

	// User saves the drawing
	canvas->Save(std::cout);
	
	// User closes the drawing
	delete canvas;
	
	// Application cleanup before shutdown
	delete vertical_ruler;
	delete horizontal_ruler;
	
	return 0;
}

