#include "rtti_shapes.h"

// Straightforward shape-derivative that draws itself, can be serialized, is movable, and has properties
class Triangle :
	public Drawable,
	public Serializable,
	public Selectable,
	public Movable,
	public Deletable
{
public:
	Triangle() :
		m_X(0),
		m_Y(0),
		m_Selected(false)
	{
	}
	
	Triangle::~Triangle()
	{
		std::cout << "Triangle::~Triangle()" << std::endl;
	}
	
	// Drawable implementation
	void Draw()
	{
		// Triangle-specific drawing code here
		std::cout << "Triangle::Draw()" << std::endl;
	}
	
	// Serializable implementation
	void Save(std::ostream& stream)
	{
		// Triangle-specific serialization code here
		std::cout << "Triangle::Save()" << std::endl;
	}
	
	// Selectable implementation
	void Select()
	{
		m_Selected = true;
	}
	
	void Deselect()
	{
		m_Selected = false;
	}
	
	bool IsSelected()
	{
		return m_Selected;
	}

	// Movable implementation
	void Move(const double X, const double Y)
	{
		m_X = X;
		m_Y = Y;
	}
	 
private:
	double m_X;
	double m_Y;
	bool m_Selected;
};

// Special shape that draws itself, can be serialized, is selectable and deletable, but not movable
class PageBorder :
	public Drawable,
	public Serializable,
	public Selectable,
	public Deletable
{
public:
	PageBorder() :
		m_Selected(false)
	{
	}
	
	~PageBorder()
	{
		std::cout << "PageBorder::~PageBorder()" << std::endl;
	}
	
	// Drawable implementation
	void Draw()
	{
		// Border-specific drawing code here
		std::cout << "PageBorder::Draw()" << std::endl;
	}
	
	// Serializable implementation
	void Save(std::ostream& Stream)
	{
		// Border-specific serialization code here
		std::cout << "PageBorder::Save()" << std::endl;
	}
	
	// Selectable implementation
	void Select()
	{
		m_Selected = true;
	}
	
	void Deselect()
	{
		m_Selected = false;
	}

	bool IsSelected()
	{
		return m_Selected;
	}
	
private:
	bool m_Selected;
};

// Application-managed shape that is Drawable only
class Ruler :
	public Drawable
{
public:
	Ruler()
	{
	}
	
	~Ruler()
	{
		std::cout << "Ruler::~Ruler()" << std::endl;
	}
	
	// Drawable implementation
	void Draw()
	{
		// Ruler-specific drawing code here
		std::cout << "Ruler::Draw()" << std::endl;
	}
};

int main(int argc, char* argv[])
{
	// Application-managed objects that aren't user-deletable
	Ruler* const vertical_ruler = new Ruler();
	Ruler* const horizontal_ruler = new Ruler();

	// Created when the user starts a new drawing
	Canvas* const canvas = new Canvas();
	
	// Default shapes that apply to every canvas
	canvas->AddShape(vertical_ruler);
	canvas->AddShape(horizontal_ruler);

	// Simulate adding  several shapes to the canvas by the user
	Unknown* const user1 = new Triangle();
	canvas->AddShape(user1);
	Unknown* const user2 = new Triangle();
	canvas->AddShape(user2);
	Unknown* const user3 = new PageBorder();
	canvas->AddShape(user3);

	// Uncomment this to test compile-time protection against assignment of objects through interfaces
	// *user1 = *user3;

	// Uncomment this to test compile-time protection against destruction of objects through interfaces
	// delete user2;
	
	// Simulate a normal screen-refresh event	
	canvas->Draw();

	// User saves the drawing
	canvas->Save(std::cout);
	
	// User closes the drawing
	delete canvas;
	
	// Application cleanup before shutdown
	delete vertical_ruler;
	delete horizontal_ruler;
	
	return 0;
}

